<?php
// $Id$
/**
 * @file
 * On behalf implementation of Feeds mapping API for date
 */

/**
 * Implementation of hook_feeds_node_processor_targets_alter().
 *
 * @see FeedsNodeProcessor::getMappingTargets().
 *
 * @todo Only provides "end date" target if field allow it.
 */
function date_feeds_node_processor_targets_alter(&$targets, $content_type) {
  $info = content_types($content_type);
  // The $info['fields'] holds CCK fields such as date and links.
  if (isset($info['fields']) && count($info['fields'])) {
    foreach ($info['fields'] as $field_name => $field) {
      // Check to see if the field type is a date (CCK).
      if (in_array($field['type'], array('date', 'datestamp', 'datetime'))) {
        // Set the name for the source label to be the label of the field or the field name if no label exists
        $name = isset($field['widget']['label']) ? $field['widget']['label'] : $field_name;

        $targets[$field_name.'#'] = array(
          'name' => $name . ' (begin)',
          'callback' => 'date_feeds_set_target',
          'description' => t('The start date for the !name field when set from a UNIX timestamp source.', array('!date_time' => $description, '!name' => $name)),
        );
        $targets[$field_name.'#2'] = array(
          'name' => $name . ' (end)',
          'callback' => 'date_feeds_set_target',
          'description' => t('The end date for the !name field when set from a UNIX timestamp source.', array('!date_time' => $description, '!name' => $name)),
        );

        //Special target for iCal VEVENT
        $targets[$field_name.'#ical'] = array(
          'name' => $name .' (iCal)',
          'description' => t('The !name field when set from an iCal VEVENT source.', array('!name' => $name)),
          'callback' => 'date_feeds_set_target_from_ical',
        );
      }
    }
  }
}

/**
 * Callback for mapping from an iCal VEVENT source to CCK date fields.
 */
function date_feeds_set_target_from_ical(&$node, $target, $values) {

  list($field_name, $sub_filed) = split('#', $target);
  $field = isset($node->$field_name) ? $node->$field_name : array();

  //We handle multiple values, be sure that $values is an array of values
  if(!is_array($values)) {
    $values = array($values);
  } else {
    //Ok we have an array, but is it used as structured data ?
    foreach(arrays_keys($values) as $key) {
      if(!is_numeric($key)) {
        $values = array($values);
        continue;
      }
    }
  }

  foreach(array_values($values) as $delta => $value) {
    if(array_key_exists('DATE', $value)) {
      $value = $value['DATE'];
    }
    //Set the field value from
    if(isset($value['DTSTART']['datetime'])) date_feeds_set_target($node, $field_name.'#', array($delta => $value['DTSTART']['datetime']));
    if(isset($value['DTSTART']['datetime'])) date_feeds_set_target($node, $field_name.'#', array($delta => $value['DTSTART']['datetime']));
  }
}

/**
 * Callback for mapping to a CCK date field.
 *
 * @todo: Add support for the field format when parsing values (as 1st fallback)
 * @todo: Add support for common formats when parsing values on PHP < 5.3 (see _create_date_from_known_formats)
 */
function date_feeds_set_target(&$node, $target, $values) {
  //Seperate out component parts. $field_name should be set to field_date, $sub_field to date.
  list($field_name, $suffix) = split('#', $target);

  $field = content_fields($field_name, $node->type);

  //Timezone objects
  $timezone_utc = timezone_open('UTC');
  $field_timezone = timezone_open($field['timezone_db']);
  if(!$field_timezone) {
    $field_timezone = $timezone_utc;
  }

  //Initial values from the field
  $field_data = isset($node->$field_name) ? $node->$field_name : array();

  //We handle multiple values, be sure that $values is an array of values
  if(!is_array($values)) {
    $values = array($values);
  } else {
    //Ok we have an array, but is it used as structured data ?
    foreach(arrays_keys($values) as $key) {
      if(!is_numeric($key)) {
        $values = array($values);
        continue;
      }
    }
  }

  foreach(array_values($values) as $delta => $value) {
    if (!empty($value)) {
      //Read the $value as a DateTime object
      //Parse it as a timestamp
      $date = date_create("@$value", $timezone_utc);
      //TODO: 1st fallback, parse it using the field format
      //if(!$date) $date = ...
      //2nd fallback, parse it using known date formats (W3C, etc.)
      if(!$date) $date = _create_date_from_known_formats($value, $timezone_utc);
      //Everything else failed, let date_create parse the string
      if(!$date) $date = date_create($value, $timezone_utc);

      if($date) {
        //Are these value needed ?
        //$field_data[$delta][$sub_field]['date'.$suffix] = $date;
        //$field_data[$delta][$sub_field]['offset'.$suffix] = $date->getOffset();
        //$field_data[$delta][$sub_field]['timezone'.$suffix] = $field_timezone->getName();
        $date->setTimezone($field_timezone);
        $field_data[$delta]['value'.$suffix] = $date->format(date_type_format($field['type']));
      } else {
        drupal_set_message(t('Could not parse %value as a date.', array('%value' => $value)), 'error');
      }
    }
  }
  $node->$field_name = $field_data;
}

if (version_compare(PHP_VERSION, '5.3', '>=')) {
  /**
   * Tries to parse a string to a DateTime object using known date formats.
   *
   * @param $time String representing the time.
   * @param $timezone Time zone
   *
   * @return A DateTime object. Or FALSE if $value cannot be parsed using a
   * known format.
   */
  function _create_date_from_known_formats($time, $timezone) {
    //See http://php.net/manual/class.datetime.php#datetime.constants.types
    static $formats = array(
      DateTime::ATOM,
      DateTime::RSS,
      DateTime::W3C,
      DateTime::RFC3339,
      DateTime::RFC2822,
      DateTime::RFC1123,
      DateTime::RFC1036,
      DateTime::RFC850,
      DateTime::RFC822,
      DateTime::ISO8601,
      DateTime::COOKIE,
    );

    foreach($formats as $format) {
      if($date = DateTime::createFromFormat($format, $value, $timezone)) {
        return $date;
      }
    }
    return FALSE;
  }
} else {
  /**
   * @todo
   *
   * @param $time String representing the time.
   * @param $timezone Time zone
   *
   * @return A DateTime object. Or FALSE if $value cannot be parsed using a
   * known format.
   */
  function _create_date_from_known_formats($time, $timezone) {
    return FALSE;
  }
}
